MAIN_CONTROLLER: str = "__main__"

LEFT_FRONT_WHEEL_SPEED_SENSOR: str = "lfwss"
RIGHT_FRONT_WHEEL_SPEED_SENSOR: str = "rfwss"
LEFT_REAR_WHEEL_SPEED_SENSOR: str = "lrwss"
RIGHT_REAR_WHEEL_SPEED_SENSOR: str = "rrwss"
MIDDLE_REAR_WHEEL_SPEED_SENSOR: str = "mrwss"



from time import time as _time

from RPi import GPIO as _GPIO

from leads import Device as _Device
from leads.data_persistence import DataPersistence as _DataPersistence


class WheelSpeedSensor(_Device):
    _start: float = .0
    _elapse: _DataPersistence[float] = _DataPersistence(max_size=8)

    def reset_time(self) -> None:
        self._start = _time()

    def pulse(self) -> None:
        self._elapse.append(_time() - self._start)
        self.reset_time()

    async def initialize(self, *parent_tags: str) -> None:
        self.pins_check(1)
        _GPIO.setup(self._pins[0], _GPIO.IN, pull_up_down=_GPIO.PUD_UP)
        _GPIO.add_event_detect(self._pins[0], _GPIO.FALLING, callback=self.pulse, bouncetime=20)
        self.reset_time()

    def read(self) -> float:
        return .0



from json import loads
from os import mkdir
from os.path import abspath, exists
from typing import Any

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from leads.comm import *
from leads.data_persistence import *
from leads_dashboard import *

config = load_config(abspath(__file__)[:-6] + "config.json")
if not exists(config.data_dir):
    mkdir(config.data_dir)
    print(f"Data dir \"{config.data_dir}\" created")

time_stamp_record = DataPersistence(config.data_dir + "/time_stamp.csv", persistence=config.enable_data_persistence,
                                    max_size=2000)
speed_record = DataPersistence(config.data_dir + "/speed.csv", persistence=config.enable_data_persistence,
                               max_size=2000)


class CustomCallback(Callback):
    def on_connect(self, service: Service, connection: Connection) -> None:
        print("Connected")

    def on_fail(self, service: Service, error: Exception) -> None:
        print(error)

    def on_receive(self, service: Service, msg: bytes) -> None:
        data = loads(msg.decode())
        time_stamp_record.append(data["t"])
        speed_record.append(data["front_wheel_speed"])

    def on_disconnect(self, service: Service, connection: Connection) -> None:
        time_stamp_record.close()
        speed_record.close()


client = start_client(config.comm_addr, create_client(config.comm_port, CustomCallback()), True)

app = FastAPI(title="LEADS VeC Remote Controller")

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["GET"],
    allow_headers=["*"],
)


@app.get("/")
async def index() -> str:
    return "LEADS VeC Remote Controller"


@app.get("/current")
async def current() -> dict[str, Any]:
    return {"t": time_stamp_record[-1], "speed": speed_record[-1]}


@app.get("/time_stamp")
async def time_stamp() -> list[int]:
    return time_stamp_record.to_list()


@app.get("/speed")
async def speed() -> list[float]:
    return speed_record.to_list()



from argparse import ArgumentParser as _ArgumentParser
from os import system as _system, chdir as _chdir
from os.path import abspath as _abspath
from sys import exit as _exit

from leads_dashboard import get_system_platform as _get_system_platform

if __name__ == '__main__':
    parser = _ArgumentParser(prog="LEADS VeC RC",
                             description="Lightweight Embedded Assisted Driving System VeC Remote Controller",
                             epilog="GitHub: https://github.com/ProjectNeura/LEADS")
    parser.add_argument("-r", "--register", choices=("systemd", "config"), default=None, help="service to register")
    args = parser.parse_args()
    if args.register == "systemd":
        if _get_system_platform() != "linux":
            _exit("Error: Unsupported operating system")
        from ._bootloader import create_service

        create_service()
        print("Service registered")
    _chdir(_abspath(__file__)[:-12])
    _system("uvicorn cli:app --reload")



from RPi import GPIO as _GPIO

from leads import Controller as _Controller, controller as _controller, MAIN_CONTROLLER as _MAIN_CONTROLLER


@_controller(_MAIN_CONTROLLER)
class RaspberryPi4B(_Controller):
    async def initialize(self, *parent_tags: str) -> None:
        _GPIO.setmode(_GPIO.BCM)

    def read(self) -> None:
        return None



from os import chmod as _chmod
from os.path import abspath as _abspath


def create_service() -> None:
    _chmod(script := _abspath(__file__)[:-10] + "leads_vec_rc.service.sh", 777)
    with open("/etc/systemd/system/leads_vec_rc.service", "w") as f:
        f.write(
            "[Unit]"
            "Description=LEADS VeC RC"
            "[Service]"
            f"ExecStart=/bin/bash {script}"
            "[Install]"
            "WantedBy=default.target"
        )



from abc import abstractmethod as _abstractmethod, ABCMeta as _ABCMeta
from typing import Any as _Any

from leads.controller.device import Device


class Controller(Device, metaclass=_ABCMeta):
    def __init__(self) -> None:
        super().__init__()
        self._devices: dict[str, Device] = {}

    def level(self) -> int:
        return len(self._parent_tags)

    async def _attach_device(self, tag: str, device: Device) -> None:
        self._devices[tag] = device
        device.tag(tag)
        await device.initialize(*tuple(self._parent_tags), self._tag)

    async def device(self, tag: str, device: Device | None = None) -> Device | None:
        if device:
            await self._attach_device(tag, device)
        else:
            return self._devices[tag]

    @_abstractmethod
    def read(self) -> _Any:
        raise NotImplementedError

    def write(self, payload: _Any) -> None:
        raise NotImplementedError



from importlib.util import find_spec as _find_spec

if not _find_spec("RPi"):
    raise ImportError("Please install `RPi.GPIO` to run this module\n>>>pip install RPi.GPIO")
if not _find_spec("serial"):
    raise ImportError("Please install `pyserial` to run this module\n>>>pip install pyserial")
